/* Inspired by https://yihui.org/en/2023/09/snap-slides/ */

*, *:before, *:after {
    box-sizing: inherit;
}

.slide {
    padding: 0.5em;
    position: relative;
    min-height: 100vh;
    scroll-snap-align: start;
}

.slide > h2, .slide > h3 {
    margin-top: unset;
}

.page-number {
    opacity: .5;
    position: absolute;
    bottom: 0;
    right: 0;
}

.overview {
    font-size: 1em;
}

.overview .slide {
    min-height: unset;
    scroll-snap-align: unset;
}

.overview .slide-container {
    display: flex;
    flex-wrap: wrap;
    justify-content: space-evenly;
}

.overview .slide-container .slide {
    width: var(--slide-width);
    border: 1px dotted #ccc;
    margin-bottom: 0.5em;
}

html {
    scroll-snap-type: y mandatory;
}

html:fullscreen::-webkit-scrollbar {
    display: none;
}

html:fullscreen {
    -ms-overflow-style: none;
    scrollbar-width: none;
}

body {
    max-width: 60rem;
    margin: auto;
    padding: 0;
    line-height: 1.5;
    box-sizing: border-box;
    font-family: sans-serif;
    font-size: 175%;
    border: 1px solid lightgray;
}

pre {
    white-space: pre-wrap;
    word-break: break-all;
}

pre code {
    display: block;
    overflow-x: auto;
}

img[src$=".svg"] {
    transform: scale(1.3);
    margin-top: 5%;
    margin-bottom: 5%;
}
