<!DOCTYPE html>
<html>
  <head>
    <title></title>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <link rel="icon" type="image/x-icon" href="@root/static/favicon.ico">
    {% if css_file %}<link rel="stylesheet" href="@root/static/{{css_file}}" type="text/css">{% endif %}
    <link rel="stylesheet" href="@root/static/page.css" type="text/css">
  </head>
  <body>
    <nav>
      <div class="row">
	<div class="col-2 left">
	  <a href="@root/">Home</a>
	</div>
	<div class="col-10 right">
	  <a href="@root/license.html">License</a>
	  &middot;
	  <a href="@root/bibliography.html">Bibliography</a>
	  &middot;
	  <a href="@root/glossary.html">Glossary</a>
	</div>
      </div>
    </nav>
    <main>
{{content}}
    </main>
    <footer>
      <a href="@root/">The Webonomicon</a>
      copyright &copy; 2024
      <a href="@root/contributing.html#contributors">the authors</a>
    </footer>
  </body>
</html>
